@@ -19,7 +19,7 @@
  *
  */ 
 
-#include <libicq2000/ICQ.h>
+#include "ICQ.h"
 
 #include "sstream_fix.h"
 #include <memory>
@@ -82,6 +82,9 @@ namespace ICQ2000 {
     case MSG_Type_AutoReq_FFC:
       ist = new AwayMsgSubType(type);
       break;
+    case MSG_Type_FT:
+      ist = new FTICQSubType();
+      break;	 
     default:
       {
 	ostringstream ostr;
@@ -218,14 +221,14 @@ namespace ICQ2000 {
 
   void UINICQSubType::OutputBodyUINACK(Buffer& b) const
   {
-    b.PackUint16TranslatedNull(m_away_message);
+    b.PackUint16StringNull(m_away_message);
   }
 
   void UINICQSubType::ParseBodyUINACK(Buffer& b)
   {
     /* whatever the message type, ACKs are always the same
        - the current away message */
-    b.UnpackUint16TranslatedNull(m_away_message);
+    b.UnpackUint16StringNull(m_away_message);
   }
 
   NormalICQSubType::NormalICQSubType(bool multi)
@@ -244,7 +247,6 @@ namespace ICQ2000 {
 
   void NormalICQSubType::ParseBodyUIN(Buffer& b) {
     b.UnpackUint16StringNull(m_message);
-    b.ServerToClient(m_message);
 
     if (m_advanced) {
       b >> m_foreground
@@ -256,7 +258,7 @@ namespace ICQ2000 {
   }
 
   void NormalICQSubType::OutputBodyUIN(Buffer& b) const {
-    b.PackUint16TranslatedNull(m_message);
+    b.PackUint16StringNull(m_message);
 
     if (m_advanced) {
       b << (unsigned int)m_foreground
@@ -266,7 +268,7 @@ namespace ICQ2000 {
 
   void NormalICQSubType::ParseBodyUINACK(Buffer& b)
   {
-    b.UnpackUint16TranslatedNull(m_away_message);
+    b.UnpackUint16StringNull(m_away_message);
     b.advance(8);
   }
 
@@ -274,18 +276,16 @@ namespace ICQ2000 {
   {
     /* hmmff.. Normal messages differ from all the other ACKs by
        having these two pointless ints. */
-    b.PackUint16TranslatedNull(m_away_message);
+    b.PackUint16StringNull(m_away_message);
     if (m_advanced) {
       b << (unsigned int)0x00000000
 	<< (unsigned int)0xffffffff;
     }
   }
 
-  unsigned short NormalICQSubType::Length() const {
-    string text = m_message;
-    Translator::LFtoCRLF(text);
-
-    return text.size() + (m_advanced ? 13 : 5);
+  unsigned short NormalICQSubType::Length() const
+  {
+    return m_message.size() + (m_advanced ? 13 : 5);
   }
 
   unsigned char NormalICQSubType::getType() const { return MSG_Type_Normal; }
@@ -327,8 +327,7 @@ namespace ICQ2000 {
       m_message = text;
       m_url = "";
     }
-    b.ServerToClient(m_message);
-    b.ServerToClient(m_url);
+
 
   }
 
@@ -340,19 +339,15 @@ namespace ICQ2000 {
 
       string message = m_message;
       string url = m_url;
-      b.ClientToServer(message);
-      b.ClientToServer(url);     // I think url should be translated too ??
       ostr << message << (unsigned char)0xfe << url;
 
       b.PackUint16StringNull( ostr.str() );
     }
   }
 
-  unsigned short URLICQSubType::Length() const {
-    string text = m_message + m_url;
-    Translator::LFtoCRLF(text);
-
-    return text.size() + 6;
+  unsigned short URLICQSubType::Length() const
+  {
+    return m_message.size() + 6;
   }
 
   unsigned char URLICQSubType::getType() const { return MSG_Type_URL; }
@@ -578,22 +573,22 @@ namespace ICQ2000 {
     string_split( text, string("\xfe"), 6, fields);
 
     list<string>::iterator iter = fields.begin();
-    m_alias = b.ServerToClientCC(*(iter++));
-    m_firstname = b.ServerToClientCC(*(iter++));
-    m_lastname = b.ServerToClientCC(*(iter++));
-    m_email = b.ServerToClientCC(*(iter++));
-    m_auth = ((*(iter++)) == "1");
-    m_message = b.ServerToClientCC(*(iter++));
+    m_alias     = *(iter++);
+    m_firstname = *(iter++);
+    m_lastname  = *(iter++);
+    m_email     = *(iter++);
+    m_auth      = ((*(iter++)) == "1");
+    m_message   = *(iter++);
   }
 
   void AuthReqICQSubType::OutputBodyUIN(Buffer& b) const {
     ostringstream ostr;
-    ostr << b.ClientToServerCC(m_alias) << (unsigned char)0xfe
-	 << b.ClientToServerCC(m_firstname) << (unsigned char)0xfe
-	 << b.ClientToServerCC(m_lastname) << (unsigned char)0xfe
-	 << b.ClientToServerCC(m_email) << (unsigned char)0xfe
+    ostr << m_alias              << (unsigned char)0xfe
+	 << m_firstname          << (unsigned char)0xfe
+	 << m_lastname           << (unsigned char)0xfe
+	 << m_email              << (unsigned char)0xfe
 	 << (m_auth ? "1" : "0") << (unsigned char)0xfe
-	 << b.ClientToServerCC(m_message);
+	 << m_message;
     
     b.PackUint16StringNull( ostr.str() );
   }
@@ -616,11 +611,10 @@ namespace ICQ2000 {
 
   void AuthRejICQSubType::ParseBodyUIN(Buffer& b) {
     b.UnpackUint16StringNull(m_message);
-    b.ServerToClient(m_message);
   }
 
   void AuthRejICQSubType::OutputBodyUIN(Buffer& b) const {
-    b.PackUint16TranslatedNull(m_message);
+    b.PackUint16StringNull(m_message);
   }
 
   unsigned short AuthRejICQSubType::Length() const {
@@ -662,12 +656,12 @@ namespace ICQ2000 {
     string_split( text, string("\xfe"), 6, fields);
 
     list<string>::iterator iter = fields.begin();
-    m_sender = b.ServerToClientCC(*(iter++));
+    m_sender  = *(iter++);
     ++iter; // ???
     ++iter; // ???
-    m_email = b.ServerToClientCC(*(iter++));
+    m_email   = *(iter++);
     ++iter; // ???
-    m_message = b.ServerToClientCC(*(iter++));
+    m_message = *(iter++);
   }
 
   void EmailExICQSubType::OutputBody(Buffer& b) const {
@@ -701,12 +695,12 @@ namespace ICQ2000 {
     string_split( text, string("\xfe"), 6, fields);
 
     list<string>::iterator iter = fields.begin();
-    m_sender = b.ServerToClientCC(*(iter++));
+    m_sender  = *(iter++);
     ++iter; // ???
     ++iter; // ???
-    m_email = b.ServerToClientCC(*(iter++));
+    m_email   = *(iter++);
     ++iter; // ???
-    m_message = b.ServerToClientCC(*(iter++));
+    m_message = *(iter++);
   }
 
   void WebPagerICQSubType::OutputBody(Buffer& b) const {
@@ -755,19 +749,19 @@ namespace ICQ2000 {
     string_split( text, string("\xfe"), 5, fields);
 
     list<string>::iterator iter = fields.begin();
-    m_alias = b.ServerToClientCC(*(iter++));
-    m_firstname = b.ServerToClientCC(*(iter++));
-    m_lastname = b.ServerToClientCC(*(iter++));
-    m_email = b.ServerToClientCC(*(iter++));
-    m_auth = ((*(iter++)) == "1");
+    m_alias     = *(iter++);
+    m_firstname = *(iter++);
+    m_lastname  = *(iter++);
+    m_email     = *(iter++);
+    m_auth      = ((*(iter++)) == "1");
   }
 
   void UserAddICQSubType::OutputBodyUIN(Buffer& b) const {
     ostringstream ostr;
-    ostr << b.ClientToServerCC(m_alias) << (unsigned char)0xfe
-	 << b.ClientToServerCC(m_firstname) << (unsigned char)0xfe
-	 << b.ClientToServerCC(m_lastname) << (unsigned char)0xfe
-	 << b.ClientToServerCC(m_email) << (unsigned char)0xfe
+    ostr << m_alias              << (unsigned char)0xfe
+	 << m_firstname          << (unsigned char)0xfe
+	 << m_lastname           << (unsigned char)0xfe
+	 << m_email              << (unsigned char)0xfe
 	 << (m_auth ? "1" : "0") << (unsigned char)0xfe;
     
     b.PackUint16StringNull( ostr.str() );
@@ -777,6 +771,124 @@ namespace ICQ2000 {
 
   unsigned char UserAddICQSubType::getType() const { return MSG_Type_UserAdd; }
 
+// ============================================================================
+//  File Transfer request message
+// ============================================================================
+
+  FTICQSubType::FTICQSubType()
+    : m_port(0), m_revport(0), m_size(0)
+  { }
+
+  FTICQSubType::FTICQSubType(const std::string& msg, const std::string& desc, const int size)
+    : m_port(0), m_revport(0), m_size(size), m_message(msg),
+      m_description(desc)
+  { }
+	
+  void FTICQSubType::ParseBodyUIN(Buffer& b)
+  {
+    b.UnpackUint16StringNull(m_message);
+    b >> m_revport;   // network order
+    b.advance(2);     // unknown
+    b.UnpackUint16StringNull(m_description);
+    b >> m_size;      // total size of transfer
+    b >> m_port;
+    b.advance(2);     // unknown
+  }
+	
+  void FTICQSubType::ParseBodyUINACK(Buffer& b)
+  { 
+    b.UnpackUint16StringNull(m_message);
+    b >> m_revport;   // network order
+    b.advance(2);     // unknown
+    b.UnpackUint16StringNull(m_description);
+    b >> m_size;      // total size of transfer
+    b >> m_port;
+    b.advance(2);     // unknown
+  }
+	
+  void FTICQSubType::OutputBodyUIN(Buffer& b) const
+  {
+    b.PackUint16StringNull(m_message);
+    b << m_revport;
+    b << (unsigned short)0;
+    b.PackUint16StringNull(m_description);
+    b << m_size;
+    b << m_port;
+    b << (unsigned short)0;
+  }
+  
+  void FTICQSubType::OutputBodyUINACK(Buffer& b) const
+  {
+    b.PackUint16StringNull(m_message);
+    b << m_revport;
+    b << (unsigned short)0;
+    b.PackUint16StringNull(m_description);
+    b << m_size;
+    b << m_port;
+    b << (unsigned short)0;
+  }
+  
+  unsigned short FTICQSubType::Length() const 
+  {
+    return 0;
+  }
+
+  unsigned char FTICQSubType::getType() const
+  {
+    return MSG_Type_FT;
+  }
+
+  unsigned int FTICQSubType::getSize() const
+  {
+    return m_size;
+  }
+  
+  unsigned short FTICQSubType::getPort() const
+  {
+    return m_port;
+  }
+  
+  unsigned short FTICQSubType::getRevPort() const
+  {
+    return ( ((m_revport & 0xFF) << 8) + ((m_revport >> 8) &0xFF) );
+  }
+
+  
+  std::string FTICQSubType::getMessage() const
+  {
+    return m_message;
+  }
+  
+  std::string FTICQSubType::getDescription() const
+  {
+    return m_description;
+  }
+
+  void FTICQSubType::setMessage(const string& msg)
+  {
+    m_message = msg;
+  }
+  
+  void FTICQSubType::setDescription(const string& desc)
+  {
+    m_description = desc;
+  }
+  
+  void FTICQSubType::setPort(unsigned short port)
+  {
+    m_port = port;
+  }
+  
+  void FTICQSubType::setRevPort(unsigned short port)
+  {
+    m_revport = ((port >> 8) & 0xFF) + ((port & 0xFF) << 8);
+  }
+ 
+  void FTICQSubType::setSize(unsigned int size)
+  {
+    m_size = size;
+  }
+	
   ContactICQSubType::ContactICQSubType() { }
 
   ContactICQSubType::ContactICQSubType(const std::list<ContactRef> &content)
@@ -795,10 +907,14 @@ namespace ICQ2000 {
       list<string> fields;
       string_split(text, string("\xfe"), n, fields);
 
+	  // handle odd number of fields (last one is 0) in offline contact messages (possibly only when sent from Mac ICQ)
+	  if ( 0!=fields.size()%2 )
+		fields.pop_back();
+		
       list<string>::iterator iter = fields.begin();
       while(iter != fields.end()) {
 	ContactRef ct(new Contact(atoi((iter++)->c_str())));
-	ct->setAlias(b.ServerToClientCC(*(iter++)));
+	ct->setAlias(*(iter++));
 	m_content.push_back(ct);
       }
     }
