@@ -19,7 +19,7 @@
  *
  */
 
-#include <libicq2000/TLV.h>
+#include "TLV.h"
 
 using std::string;
 using std::map;
@@ -161,6 +161,33 @@ namespace ICQ2000 {
 	break;
       }
       break;
+
+      // ----- SBLDATA -----
+    case TLV_ParseMode_SBL:
+      switch(type) {
+      case TLV_SBL_Await_Auth:
+	tlv = new SBLAwaitAuthTLV();
+	break;
+      case TLV_SBL_Ids:
+	tlv = new SBLIdsTLV();
+	break;
+      case TLV_SBL_Visibility:
+	tlv = new SBLVisibilityTLV();
+	break;
+      case TLV_SBL_ICQTIC:
+	tlv = new SBLICQTICTLV();
+	break;
+      case TLV_SBL_ImportTime:
+	tlv = new SBLImportTimeTLV();
+	break;
+      case TLV_SBL_Nick:
+	tlv = new SBLNickTLV();
+	break;
+      case TLV_SBL_SMS_No:
+	tlv = new SBLSMSNoTLV();
+	break;
+      }
+      break;
     }
 
     if (tlv == NULL) {
@@ -186,6 +213,18 @@ namespace ICQ2000 {
 
   // ----------------- Base Classes ---------------
 
+  CharTLV::CharTLV() { }
+  CharTLV::CharTLV(unsigned char n) : m_value(n) { }
+  void CharTLV::OutputValue(Buffer& b) const {
+    b << Length();
+    b << m_value;
+  }
+  void CharTLV::ParseValue(Buffer& b) {
+    unsigned short l;
+    b >> l; // should be 1
+    b >> m_value;
+  }
+
   ShortTLV::ShortTLV() { }
   ShortTLV::ShortTLV(unsigned short n) : m_value(n) { }
   void ShortTLV::OutputValue(Buffer& b) const {
@@ -427,7 +466,6 @@ namespace ICQ2000 {
     b >> m_flag2;
 
     b.Unpack(m_message, length-4);
-    b.ServerToClient(m_message);
   }
 
   AdvMsgDataTLV::AdvMsgDataTLV() : m_icqsubtype(NULL) { }
@@ -511,6 +549,28 @@ namespace ICQ2000 {
     }
   }
 
+  // ============================================================================
+  //  SBL TLVs
+  // ============================================================================
+
+  void SBLAwaitAuthTLV::ParseValue(Buffer& b)
+  {
+    unsigned short l;
+    b >> l; // should be 0
+    b.advance(l);
+  }
+  
+  void SBLIdsTLV::ParseValue(Buffer& b)
+  {
+    unsigned short l;
+    b >> l;
+    b.advance(l); // TODO
+  }
+
+  // ============================================================================
+  //  ICQ TLVs
+  // ============================================================================
+
   ICQDataTLV::ICQDataTLV() : m_icqsubtype(NULL) { }
 
   ICQDataTLV::~ICQDataTLV() {
@@ -560,7 +620,8 @@ namespace ICQ2000 {
     tlvmap.clear();
   }
 
-  void TLVList::Parse(Buffer& b, TLV_ParseMode pm, unsigned short no_tlvs) {
+  void TLVList::Parse(Buffer& b, TLV_ParseMode pm, unsigned short no_tlvs)
+  {
     InTLV *t;
     unsigned short ntlv = 0;
     while (b.beforeEnd() && ntlv < no_tlvs) {
@@ -575,6 +636,21 @@ namespace ICQ2000 {
     }
   }
 
+  void TLVList::ParseByLength(Buffer& b, TLV_ParseMode pm, unsigned int len) 
+  {
+    InTLV *t;
+    unsigned int end = b.pos() + len;
+    while (b.pos() < end) {
+      t = InTLV::ParseTLV(b,pm);
+      // duplicate TLVs of one type - this shouldn't happen!
+      if (tlvmap.count(t->Type())) {
+	delete tlvmap[t->Type()];
+      }
+
+      tlvmap[t->Type()] = t;
+    }
+  }
+
   bool TLVList::exists(unsigned short type) {
     return (tlvmap.count(type) != 0);
   }
@@ -583,6 +659,7 @@ namespace ICQ2000 {
     return tlvmap[type];
   }
 
+  Buffer& operator<<(Buffer& b, const ICQ2000::OutTLV& tlv) { tlv.Output(b); return b; }
+
 }
 
-Buffer& operator<<(Buffer& b, const ICQ2000::OutTLV& tlv) { tlv.Output(b); return b; }
