@@ -1,7 +1,7 @@
 /*
  * libyahoo2: yahoo_httplib.c
  *
- * Copyright (C) 2002, Philip S Tellis <philip . tellis AT gmx . net>
+ * Copyright (C) 2002-2004, Philip S Tellis <philip.tellis AT gmx.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,12 +25,26 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
+
+#if STDC_HEADERS
+# include <string.h>
+#else
+# if !HAVE_STRCHR
+#  define strchr index
+#  define strrchr rindex
+# endif
+char *strchr (), *strrchr ();
+# if !HAVE_MEMCPY
+#  define memcpy(d, s, n) bcopy ((s), (d), (n))
+#  define memmove(d, s, n) bcopy ((s), (d), (n))
+# endif
+#endif
+
 
 #include <errno.h>
 #include <unistd.h>
 #include <ctype.h>
-#include "libyahoo2.h"
+#include "yahoo2.h"
 #include "yahoo2_callbacks.h"
 #include "yahoo_httplib.h"
 #include "yahoo_util.h"
@@ -45,9 +59,9 @@
 
 #ifdef USE_STRUCT_CALLBACKS
 extern struct yahoo_callbacks *yc;
-#define YAHOO_CALLBACK(x)       yc->x
+#define YAHOO_CALLBACK(x)	yc->x
 #else
-#define YAHOO_CALLBACK(x)       x
+#define YAHOO_CALLBACK(x)	x
 #endif
 
 extern enum yahoo_log_level log_level;
@@ -61,10 +75,10 @@ int yahoo_tcp_readline(char *ptr, int maxlen, int fd)
 
 		do {
 			rc = read(fd, &c, 1);
-		} while(rc == -1 && errno == EINTR);
+		} while(rc == -1 && (errno == EINTR || errno == EAGAIN)); /* this is bad - it should be done asynchronously */
 
 		if (rc == 1) {
-			if(c == '\r')                   /* get rid of \r */
+			if(c == '\r')			/* get rid of \r */
 				continue;
 			*ptr = c;
 			if (c == '\n')
@@ -72,9 +86,9 @@ int yahoo_tcp_readline(char *ptr, int maxlen, int fd)
 			ptr++;
 		} else if (rc == 0) {
 			if (n == 1)
-				return (0);             /* EOF, no data */
+				return (0);		/* EOF, no data */
 			else
-				break;                  /* EOF, w/ data */
+				break;			/* EOF, w/ data */
 		} else {
 			return -1;
 		}
@@ -244,7 +258,7 @@ char *yahoo_xmldecode(const char *instr)
 			for (i=0; i<5; i++) 
 				if(!strncmp(instr+ipos, entitymap[i][0], 
 					       strlen(entitymap[i][0]))) {
-					str[bpos++] = entitymap[i][1][0];
+				       	str[bpos++] = entitymap[i][1][0];
 					ipos += strlen(entitymap[i][0]);
 					break;
 				}
@@ -348,6 +362,7 @@ static void yahoo_got_url_fd(int id, int fd, int error, void *data)
 	char buff[1024];
 	unsigned long filesize=0;
 	char *filename=NULL;
+	int n;
 
 	struct url_data *ud = data;
 
@@ -357,8 +372,8 @@ static void yahoo_got_url_fd(int id, int fd, int error, void *data)
 		return;
 	}
 
-	while(yahoo_tcp_readline(buff, sizeof(buff), fd) > 0) {
-		/* read up to blank line */
+	while((n=yahoo_tcp_readline(buff, sizeof(buff), fd)) > 0) {
+		LOG(("Read:%s:\n", buff));
 		if(!strcmp(buff, ""))
 			break;
 
@@ -396,6 +411,8 @@ static void yahoo_got_url_fd(int id, int fd, int error, void *data)
 		}
 	}
 
+	LOG(("n == %d\n", n));
+	LOG(("Calling callback, filename:%s, size: %ld\n", filename, filesize));
 	ud->callback(id, fd, error, filename, filesize, ud->user_data);
 	FREE(ud);
 	FREE(filename);
