@@ -1,7 +1,7 @@
 /*
 *
 * centericq HTTP protocol handling class
-* $Id: HTTPClient.cc,v 1.9 2003/10/19 23:24:35 konst Exp $
+* $Id: HTTPClient.cc,v 1.12 2004/02/10 23:55:16 konst Exp $
 *
 * Copyright (C) 2003 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -26,7 +26,7 @@
 
 #ifdef BUILD_RSS
 
-HTTPClient::HTTPClient() : m_recv(&m_transl), m_state(NOT_CONNECTED),
+HTTPClient::HTTPClient() : m_recv(), m_state(NOT_CONNECTED),
 m_timeout(50), m_proxy_port(8080) {
     m_socket = new TCPSocket();
     Init();
@@ -188,10 +188,43 @@ void HTTPClient::Send(Buffer &b) {
     }
 }
 
+static char base64digits[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+			     "abcdefghijklmnopqrstuvwxyz"
+			     "0123456789._";
+
+string HTTPClient::base64_encode(const string in, int inlen) {
+    string out;
+    int j = 0;
+
+    for (; inlen >= 3; inlen -= 3) {
+	out += base64digits[ in[j] >> 2 ];
+	out += base64digits[ ((in[j] << 4) & 0x30) | (in[j+1] >> 4) ];
+	out += base64digits[ ((in[j+1] << 2) & 0x3c) | (in[j+2] >> 6) ];
+	out += base64digits[ in[j+2] & 0x3f ];
+	j += 3;
+    }
+
+    if (inlen > 0) {
+	unsigned char fragment;
+
+	out += base64digits[in[j] >> 2];
+	fragment = (in[j] << 4) & 0x30;
+
+	if (inlen > 1)
+	    fragment |= in[j+1] >> 4;
+
+	out += base64digits[fragment];
+	out += (inlen < 2) ? '-' : base64digits[ (in[j+1] << 2) & 0x3c ];
+	out += '-';
+    }
+    
+    return out;
+}
+
 void HTTPClient::SendRequest() {
-    Buffer b(&m_transl);
+    Buffer b;
     HTTPRequestEvent *ev = m_queue.front();
-    string rq;
+    string rq, userpasswd;
 
     switch(ev->getMethod()) {
 	case HTTPRequestEvent::GET: rq = "GET"; break;
@@ -205,6 +238,14 @@ void HTTPClient::SendRequest() {
 	b.Pack(rq + " http://" + m_hostname + m_resource + " HTTP/1.0\r\n");
     }
 
+    if (!m_proxy_user.empty()) {
+	userpasswd = (string)m_proxy_user + ":" + m_proxy_passwd;
+
+	if (!(m_proxy_hostname.empty())) {
+	    b.Pack((string) "Proxy-Authorization: Basic " + base64_encode(userpasswd, userpasswd.size()) + "\r\n");
+	};
+    }
+
     b.Pack((string) "User-Agent: " + PACKAGE + "/" + VERSION + "\r\n");
 
     if(ev->getMethod() == HTTPRequestEvent::POST) {
@@ -285,7 +326,7 @@ void HTTPClient::check_timeout() {
 	MessageEvent *ev = *m_queue.begin();
 	ev->setDelivered(false);
 	ev->setFinished(true);
-	dynamic_cast<HTTPRequestEvent*>(ev)->setHTTPResp(gettext("Timed out"));
+	dynamic_cast<HTTPRequestEvent*>(ev)->setHTTPResp("Timed out");
 	ev->setDeliveryFailureReason(MessageEvent::Failed);
 	messageack.emit(ev);
 
