@@ -1,7 +1,7 @@
 /*
 *
 * centericq IRC protocol handling class
-* $Id: irchook.cc,v 1.74 2003/11/05 09:07:42 konst Exp $
+* $Id: irchook.cc,v 1.77 2004/01/27 00:43:30 konst Exp $
 *
 * Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -126,13 +126,13 @@ void irchook::init() {
     firetalk_subcode_register_reply_callback(handle, "VERSION", &subreply);
 
 #ifdef DEBUG
-    firetalk_register_callback(handle, FC_LOG, &log);
+    firetalk_register_callback(handle, FC_LOG, &fclog);
 #endif
 }
 
 void irchook::connect() {
     icqconf::imaccount acc = conf.getourid(irc);
-    face.log(_("+ [irc] connecting to the server"));
+    log(logConnecting);
 
     firetalk_register_callback(handle, FC_DISCONNECT, 0);
     firetalk_disconnect(handle);
@@ -273,6 +273,9 @@ bool irchook::send(const imevent &ev) {
 	    } else if(cmd == "DEOP") {
 		text = (string) "mode " + c->getdesc().nickname + " -o " + text;
 
+	    } else if(cmd == "RAW") {
+		// leave text intact
+
 	    } else {
 		text.insert(0, cmd + " ");
 
@@ -528,9 +531,7 @@ void irchook::processnicks() {
     }
 
     face.findready();
-
-    face.log(_("+ [irc] channel members list fetching finished, %d found"),
-	foundguys.size());
+    log(logConfMembers, foundguys.size());
 
     searchdest->redraw();
     searchdest = 0;
@@ -719,7 +720,7 @@ void irchook::userstatus(const string &nickname, imstatus st) {
 
 void irchook::connected(void *conn, void *cli, ...) {
     irhook.flogged = true;
-    face.log(_("+ [irc] logged in"));
+    irhook.log(logLogged);
     face.update();
 
     int i;
@@ -757,7 +758,7 @@ void irchook::disconnected(void *conn, void *cli, ...) {
     for(ic = irhook.channels.begin(); ic != irhook.channels.end(); ++ic)
 	ic->fetched = false;
 
-    face.log(_("+ [irc] disconnected"));
+    irhook.log(logDisconnected);
     face.update();
 }
 
@@ -888,7 +889,7 @@ void irchook::getmessage(void *conn, void *cli, ...) {
 	}
 
 	em.store(immessage(imcontact(sender, irc),
-	    imevent::incoming, irhook.rushtmlconv("wk", cuthtml(message, true))));
+	    imevent::incoming, irhook.rushtmlconv("wk", cuthtml(message, chCutBR | chLeaveLinks))));
     }
 
     DLOG("getmessage");
@@ -907,7 +908,7 @@ void irchook::getaction(void *conn, void *cli, ...) {
     if(strlen(sender) && strlen(message)) {
 	em.store(immessage(imcontact(sender, irc), imevent::incoming, 
 	    ((string) "* " + sender + " " + 
-		irhook.rushtmlconv("wk", cuthtml(message, true))).c_str()));
+		irhook.rushtmlconv("wk", cuthtml(message, chCutBR | chLeaveLinks))).c_str()));
     }
     
     DLOG("getaction");
@@ -980,7 +981,7 @@ void irchook::listmember(void *connection, void *cli, ...) {
     }
 }
 
-void irchook::log(void *connection, void *cli, ...) {
+void irchook::fclog(void *connection, void *cli, ...) {
     va_list ap;
 
     va_start(ap, cli);
@@ -1150,7 +1151,7 @@ void irchook::chataction(void *connection, void *cli, ...) {
     if(clist.get(imcontact(room, irc))) {
 	em.store(immessage(imcontact(room, irc), imevent::incoming,
 	    ((string) "* " + from + " " +
-		irhook.rushtmlconv("wk", cuthtml(msg, true))).c_str()));
+		irhook.rushtmlconv("wk", cuthtml(msg, chCutBR | chLeaveLinks))).c_str()));
     }
 }
 
@@ -1429,6 +1430,7 @@ void irchook::chatuserleft(void *conn, void *cli, ...) {
 
 	if(reason)
 	if(strlen(reason)) {
+	    if(strlen(reason) > 450) reason[450] = 0;
 	    sprintf(buf, _("reason: %s"), reason);
 	    text += (string) "; " + buf + ".";
 	}
