@@ -15,15 +15,24 @@ class ljhook: public abstracthook, public sigslot::has_slots<> {
 	vector<int> rfds, wfds, efds;
 	bool fonline, flogged;
 	imcontact self;
+	time_t timer_getfriends;
+
+	verticalmenu *sdest;
+	string lookfor;
 
 	enum RequestType {
 	    reqLogin,
 	    reqPost,
+	    reqPostComment,
 	    reqGetFriends,
+	    reqDelFriend,
+	    reqAddFriend,
+	    reqLookup,
 	    reqDone
 	};
 
 	map<HTTPRequestEvent *, RequestType> sent;
+	vector<string> friendof, journals, moods, pictures;
 
 	void socket_cb(SocketEvent *ev);
 	void messageack_cb(MessageEvent *ev);
@@ -31,6 +40,8 @@ class ljhook: public abstracthook, public sigslot::has_slots<> {
 
 	void requestfriends();
 
+	string getfeedurl(const string &nick) const;
+
     public:
 	ljhook();
 	~ljhook();
@@ -50,9 +61,9 @@ class ljhook: public abstracthook, public sigslot::has_slots<> {
 	bool isconnecting() const;
 	bool enabled() const;
 
-	bool send(const imevent &ev);
+	bool send(const imevent &sev);
 
-	void sendnewuser(const imcontact &c);
+	void sendnewuser(const imcontact &ic);
 	void removeuser(const imcontact &ic);
 
 	void setautostatus(imstatus st);
@@ -70,6 +81,10 @@ class ljhook: public abstracthook, public sigslot::has_slots<> {
 	void ouridchanged(const icqconf::imaccount &ia);
 
 	void updatecontact(icqcontact *c);
+
+	vector<string> getjournals() const { return journals; }
+	vector<string> getmoods() const { return moods; }
+	vector<string> getpictures() const { return pictures; }
 };
 
 extern ljhook lhook;
