@@ -1,9 +1,9 @@
 /*
 *
 * centericq MSN protocol handling class
-* $Id: msnhook.cc,v 1.67 2003/09/30 11:38:43 konst Exp $
+* $Id: msnhook.cc,v 1.76 2004/03/07 13:44:41 konst Exp $
 *
-* Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
+* Copyright (C) 2001-2004 by Konstantin Klyagin <konst@konst.org.ua>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -34,6 +34,7 @@
 #include "eventmanager.h"
 #include "imlogger.h"
 #include "connwrap.h"
+#include "icqgroups.h"
 
 #include "msn_bittybits.h"
 
@@ -102,10 +103,8 @@ msnhook::msnhook(): abstracthook(msn) {
     ourstatus = offline;
     fonline = false;
 
-    fcapabs.insert(hookcapab::synclist);
     fcapabs.insert(hookcapab::changedetails);
     fcapabs.insert(hookcapab::directadd);
-//    fcapabs.insert(hookcapab::files);
 }
 
 msnhook::~msnhook() {
@@ -118,13 +117,17 @@ void msnhook::init() {
 void msnhook::connect() {
     icqconf::imaccount account = conf.getourid(msn);
 
-    face.log(_("+ [msn] connecting to the server"));
+    log(logConnecting);
+
+    readinfo = flogged = false;
+    fonline = true;
 
     msn_init(&conn, nicknormalize(account.nickname).c_str(), account.password.c_str());
     msn_connect(&conn, account.server.c_str(), account.port);
 
-    fonline = true;
-    flogged = false;
+    if(conn.ready) {
+	fonline = flogged = true;
+    }
 }
 
 void msnhook::disconnect() {
@@ -136,27 +139,14 @@ void msnhook::exectimers() {
 
 void msnhook::main() {
     vector<int>::const_iterator i;
-    fd_set rs, ws;
+    fd_set rs, ws, es;
     struct timeval tv;
-    int hsock;
-
-    FD_ZERO(&rs);
-    FD_ZERO(&ws);
+    int hsock = 0;
 
+    getsockets(rs, ws, es, hsock);
     tv.tv_sec = tv.tv_usec = 0;
-    hsock = 0;
 
-    for(i = rfds.begin(); i != rfds.end(); ++i) {
-	FD_SET(*i, &rs);
-	hsock = max(hsock, *i);
-    }
-
-    for(i = wfds.begin(); i != wfds.end(); ++i) {
-	FD_SET(*i, &ws);
-	hsock = max(hsock, *i);
-    }
-
-    if(select(hsock+1, &rs, &ws, 0, &tv) > 0) {
+    if(select(hsock+1, &rs, &ws, &es, &tv) > 0) {
 	for(i = rfds.begin(); i != rfds.end(); ++i)
 	    if(FD_ISSET(*i, &rs)) {
 		msn_handle_incoming(*i, 1, 0);
@@ -248,7 +238,7 @@ bool msnhook::send(const imevent &ev) {
     }
 
     icqcontact *c = clist.get(ev.getcontact());
-    text = siconv(text, conf.getrussian(msn) ? "koi8-u" : conf.getdefcharset(), "utf8");
+    text = rusconv("ku", text);
 
     if(c)
     if(c->getstatus() != offline || !c->inlist()) {
@@ -259,8 +249,28 @@ bool msnhook::send(const imevent &ev) {
     return false;
 }
 
+int msnhook::findgroup(const imcontact &ic, string &gname) const {
+    int gid = -1;
+    icqcontact *c;
+
+    if(c = clist.get(ic)) {
+	vector<icqgroup>::const_iterator ig = find(groups.begin(), groups.end(), c->getgroupid());
+	if(ig != groups.end()) {
+	    gname = ig->getname();
+	    map<int, string>::const_iterator im = mgroups.begin();
+	    while(im != mgroups.end() && gid == -1) {
+		if(im->second == ig->getname())
+		    gid = im->first;
+		++im;
+	    }
+	}
+    }
+
+    return gid;
+}
+
 void msnhook::sendnewuser(const imcontact &ic) {
-    if(logged()) {
+    if(logged() && !readinfo) {
 	icqcontact *c;
 	imcontact icc(nicktodisp(ic.nickname), msn);
 
@@ -271,7 +281,16 @@ void msnhook::sendnewuser(const imcontact &ic) {
 	    c->setdispnick(icc.nickname);
 	}
 
-	msn_add_to_list(&conn, "FL", nicknormalize(ic.nickname).c_str());
+	int gid;
+	string gname;
+
+	gid = findgroup(ic, gname);
+
+	if(gid != -1) {
+	    msn_add_to_list(&conn, "FL", nicknormalize(ic.nickname).c_str(), gid);
+	} else {
+	    msn_add_group(&conn, gname.c_str());
+	}
     }
 
     requestinfo(ic);
@@ -301,21 +320,32 @@ void msnhook::removeuser(const imcontact &ic) {
 }
 
 void msnhook::removeuser(const imcontact &ic, bool report) {
-    if(online()) {
-	if(report)
-	    face.log(_("+ [msn] removing %s from the contacts"), ic.nickname.c_str());
+    int i;
+    bool found;
+    vector<msnbuddy>::const_iterator ib = find(slst["FL"].begin(), slst["FL"].end(), nicknormalize(ic.nickname));
 
-	msn_del_from_list(&conn, "FL", nicknormalize(ic.nickname).c_str());
+    if(online() && ib != slst["FL"].end()) {
+	if(report)
+	    log(logContactRemove, ic.nickname.c_str());
+
+	msn_del_from_list(&conn, "FL",
+	    nicknormalize(ic.nickname).c_str(),
+	    ib->gid);
+
+	for(i = 0, found = false; i < clist.count && !found; i++) {
+	    icqcontact *c = (icqcontact *) clist.at(i);
+	    found = c->getdesc().pname == msn
+		&& groups.getname(c->getgroupid()) == mgroups[ib->gid];
+	}
+
+	if(!found)
+	    msn_del_group(&conn, ib->gid);
     }
 }
 
 void msnhook::requestinfo(const imcontact &ic) {
     icqcontact *c = clist.get(ic);
-
-    if(!c) {
-	c = clist.get(contactroot);
-	c->clear();
-    }
+    if(!c) c = clist.get(contactroot);
 
     icqcontact::moreinfo m = c->getmoreinfo();
     icqcontact::basicinfo b = c->getbasicinfo();
@@ -332,13 +362,13 @@ void msnhook::requestinfo(const imcontact &ic) {
 
 void msnhook::lookup(const imsearchparams &params, verticalmenu &dest) {
     if(params.reverse) {
-	vector<pair<string, string> >::const_iterator i = slst["RL"].begin();
+	vector<msnbuddy>::const_iterator i = slst["RL"].begin();
 
 	while(i != slst["RL"].end()) {
-	    icqcontact *c = new icqcontact(imcontact(nicktodisp(i->first), msn));
-	    c->setnick(i->second);
+	    icqcontact *c = new icqcontact(imcontact(nicktodisp(i->nick), msn));
+	    c->setnick(i->friendly);
 
-	    dest.additem(conf.getcolor(cp_clist_msn), c, (string) " " + i->first);
+	    dest.additem(conf.getcolor(cp_clist_msn), c, (string) " " + i->nick);
 	    ++i;
 	}
 	face.findready();
@@ -359,10 +389,10 @@ vector<icqcontact *> msnhook::getneedsync() {
 	icqcontact *c = (icqcontact *) clist.at(i);
 
 	if(c->getdesc().pname == msn) {
-	    vector<pair<string, string> >::const_iterator fi = slst["FL"].begin();
+	    vector<msnbuddy>::const_iterator fi = slst["FL"].begin();
 
 	    for(found = false; fi != slst["FL"].end() && !found; ++fi)
-		found = c->getdesc().nickname == fi->first;
+		found = c->getdesc().nickname == fi->nick;
 
 	    if(!found)
 		r.push_back(c);
@@ -442,6 +472,30 @@ bool msnhook::getfevent(invitation_ftp *fhandle, imfile &fr) {
     return false;
 }
 
+void msnhook::updatecontact(icqcontact *c) {
+    string gname, nick = nicknormalize(c->getdesc().nickname);
+    vector<msnbuddy>::const_iterator ib = find(slst["FL"].begin(), slst["FL"].end(), nick);
+
+    if(ib != slst["FL"].end() && logged() && conf.getgroupmode() != icqconf::nogroups)
+    if(mhook.findgroup(c->getdesc(), gname) != ib->gid) {
+	msn_del_from_list(&conn, "FL", nick.c_str());
+	sendnewuser(c->getdesc());
+    }
+}
+
+void msnhook::renamegroup(const string &oldname, const string &newname) {
+    if(logged()) {
+	map<int, string>::const_iterator im = mgroups.begin();
+	while(im != mgroups.end()) {
+	    if(im->second == oldname) {
+		msn_rename_group(&conn, im->first, newname.c_str());
+		break;
+	    }
+	    ++im;
+	}
+    }
+}
+
 // ----------------------------------------------------------------------------
 
 static void log(const string &s) {
@@ -450,6 +504,11 @@ static void log(const string &s) {
 #endif
 }
 
+int ext_debug(char *str) {
+    log(str);
+    return 0;
+}
+
 void ext_register_sock(int s, int reading, int writing) {
     log("ext_register_sock");
     if(reading) mhook.rfds.push_back(s);
@@ -481,11 +540,19 @@ void ext_got_info(msnconn *conn, syncinfo *info) {
     userdata *ud;
     llist *lst, *pl;
     imcontact ic;
+    bool found;
+
+    mhook.readinfo = true;
+
+    for(pl = info->grp, mhook.mgroups.clear(); pl; pl = pl->next) {
+	groupdata *gd = (groupdata *) pl->data;
+	mhook.mgroups[gd->number] = gd->title;
+    }
 
     for(lst = info->fl; lst; lst = lst->next) {
 	ud = (userdata *) lst->data;
 
-	mhook.slst["FL"].push_back(make_pair(ud->username, ud->friendlyname));
+	mhook.slst["FL"].push_back(msnbuddy(ud->username, ud->friendlyname, ud->grp));
 
 	ic = imcontact(nicktodisp(ud->username), msn);
 	icqcontact *c = clist.get(ic);
@@ -508,14 +575,21 @@ void ext_got_info(msnconn *conn, syncinfo *info) {
 
 	c->setbasicinfo(bi);
 	c->setworkinfo(wi);
+
+	for(found = false, pl = info->grp; pl && !found; pl = pl->next) {
+	    groupdata *gd = (groupdata *) pl->data;
+	    found = gd->number == ud->grp;
+	    if(found) clist.updateEntry(ic, gd->title);
+	}
     }
 
     for(lst = info->rl; lst; lst = lst->next) {
 	ud = (userdata *) lst->data;
-	mhook.slst["RL"].push_back(make_pair(ud->username, ud->friendlyname));
+	mhook.slst["RL"].push_back(msnbuddy(ud->username, ud->friendlyname));
     }
 
     mhook.setautostatus(mhook.ourstatus);
+    mhook.readinfo = false;
 }
 
 void ext_latest_serial(msnconn * conn, int serial) {
@@ -539,17 +613,17 @@ void ext_new_RL_entry(msnconn *conn, const char *username, const char *friendlyn
     em.store(imnotification(ic, _("The user has added you to his/her contact list")));
 }
 
-void ext_new_list_entry(msnconn *conn, const char *lst, const char *username) {
+void ext_new_list_entry(msnconn *conn, const char *lst, const char *username, const char *friendlyname, int gid) {
     log("ext_new_list_entry");
-    mhook.slst[lst].push_back(make_pair(username, string()));
+    mhook.slst[lst].push_back(msnbuddy(username, friendlyname, gid));
 }
 
 void ext_del_list_entry(msnconn *conn, const char *lst, const char *username) {
     log("ext_del_list_entry");
 
-    vector<pair<string, string> >::iterator i = mhook.slst[lst].begin();
+    vector<msnbuddy>::iterator i = mhook.slst[lst].begin();
     while(i != mhook.slst[lst].end()) {
-	if(i->first == username) {
+	if(i->nick == username) {
 	    mhook.slst[lst].erase(i);
 	    i = mhook.slst[lst].begin();
 	} else {
@@ -602,7 +676,7 @@ void ext_got_IM(msnconn *conn, const char *username, const char *friendlyname, m
 
     mhook.checkinlist(ic);
 
-    string text = siconv(msg->body, "utf8", conf.getrussian(msn) ? "koi8-u" : conf.getdefcharset());
+    string text = mhook.rusconv("uk", msg->body);
     em.store(immessage(ic, imevent::incoming, text));
 }
 
@@ -687,13 +761,14 @@ void ext_new_connection(msnconn *conn) {
 	msn_sync_lists(conn, 0);
 	logger.putourstatus(msn, offline, mhook.ourstatus = mhook.manualstatus);
 	mhook.flogged = true;
-	face.log(_("+ [msn] logged in"));
+	mhook.log(abstracthook::logLogged);
 	face.update();
     }
 }
 
 void ext_closing_connection(msnconn *conn) {
     log("ext_closing_connection");
+
     if(conn->type == CONN_NS) {
 	mhook.rfds.clear();
 	mhook.wfds.clear();
@@ -701,7 +776,7 @@ void ext_closing_connection(msnconn *conn) {
 	clist.setoffline(msn);
 	mhook.fonline = false;
 	mhook.slst.clear();
-	face.log(_("+ [msn] disconnected"));
+	mhook.log(abstracthook::logDisconnected);
 	face.update();
     }
 }
@@ -710,8 +785,7 @@ void ext_changed_state(msnconn *conn, const char *state) {
     log("ext_changed_state");
 }
 
-int ext_connect_socket(const char *hostname, int port) {
-    log("ext_connect_socket");
+int ext_do_connect_socket(const char *hostname, int port, int ssl) {
     struct sockaddr_in sa;
     struct hostent *hp;
     int a, s;
@@ -732,7 +806,7 @@ int ext_connect_socket(const char *hostname, int port) {
     if((s = socket(hp->h_addrtype, SOCK_STREAM, 0)) < 0)
 	return -1;
 
-    if(cw_connect(s, (struct sockaddr *) &sa, sizeof(sa), 0) < 0) {
+    if(cw_connect(s, (struct sockaddr *) &sa, sizeof(sa), ssl) < 0) {
 	face.log(msgerr + _("verify the hostname and port"));
 	close(s);
 	return -1;
@@ -741,6 +815,16 @@ int ext_connect_socket(const char *hostname, int port) {
     return s;
 }
 
+int ext_connect_socket_ssl(const char *hostname, int port) {
+    log("ext_connect_socket_ssl");
+    return ext_do_connect_socket(hostname, port, 1);
+}
+
+int ext_connect_socket(const char *hostname, int port) {
+    log("ext_connect_socket");
+    return ext_do_connect_socket(hostname, port, 0);
+}
+
 int ext_server_socket(int port) {
     log("ext_server_socket");
     int s;
@@ -780,4 +864,33 @@ void ext_protocol_log(const char *buf, int readev, int writeev) {
     }
 }
 
+void ext_new_group(msnconn * conn, const char * name, int id) {
+    int i;
+    icqcontact *c;
+
+    mhook.mgroups[id] = name;
+
+    vector<icqgroup>::const_iterator ig = groups.begin();
+
+    while(ig != groups.end()) {
+	if(ig->getname() == name) {
+	    for(i = 0; i < clist.count; i++) {
+		c = (icqcontact *) clist.at(i);
+		if(c->getgroupid() == ig->getid())
+		    mhook.sendnewuser(c->getdesc());
+	    }
+	    break;
+	}
+	++ig;
+    }
+}
+
+void ext_renamed_group(msnconn * conn, const char * name, int id) {
+    mhook.mgroups[id] = name;
+}
+
+void ext_del_group(msnconn * conn, const char * name, int id) {
+    mhook.mgroups.erase(id);
+}
+
 #endif
