@@ -1,7 +1,7 @@
 /*
 *
 * centericq rss handling class
-* $Id: rsshook.cc,v 1.17 2003/11/05 09:07:43 konst Exp $
+* $Id: rsshook.cc,v 1.20 2004/02/01 17:52:10 konst Exp $
 *
 * Copyright (C) 2003 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -216,6 +216,10 @@ void rsshook::ping(const imcontact &ic) {
     if(!url.empty()) {
 	httpcli.setProxyServerHost(conf.gethttpproxyhost());
 	httpcli.setProxyServerPort(conf.gethttpproxyport());
+	if (!conf.gethttpproxyuser().empty()) {
+	    httpcli.setProxyServerUser(conf.gethttpproxyuser());
+	    httpcli.setProxyServerPasswd(conf.gethttpproxypasswd());
+	}
 	httpcli.SendEvent(new HTTPRequestEvent(url));
     }
 }
@@ -266,8 +270,8 @@ const string &name, const string &title, const string &postfix) {
 
     if(d)
     if(!d->getValue().empty()) {
-	val = rushtmlconv("wk", cuthtml(d->getValue(), true), false);
-	if(!enc.empty()) val = siconv(val, enc, conf.getrussian(rss) ? "koi8-u" : conf.getdefcharset());
+	val = rushtmlconv("wk", cuthtml(d->getValue(), chCutBR | chLeaveLinks), false);
+	if(!enc.empty()) val = siconv(val, enc, conf.getconvertto(rss));
 
 	while((pos = val.find("<br")) != -1 || (pos = val.find("<BR")) != -1) {
 	    if((n = val.substr(pos).find(">")) != -1) {
