@@ -1,9 +1,9 @@
 /*
 *
 * centericq configuration handling routines
-* $Id: icqconf.cc,v 1.121 2003/10/26 10:46:52 konst Exp $
+* $Id: icqconf.cc,v 1.130 2004/03/17 19:08:31 konst Exp $
 *
-* Copyright (C) 2001,2002 by Konstantin Klyagin <konst@konst.org.ua>
+* Copyright (C) 2001-2004 by Konstantin Klyagin <konst@konst.org.ua>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -54,8 +54,7 @@ icqconf::icqconf() {
 
     for(protocolname pname = icq; pname != protocolname_size; (int) pname += 1) {
 	chatmode[pname] = true;
-	russian[pname] = false;
-	entersends[pname] = false;
+	cpconvert[pname] = entersends[pname] = nonimonline[pname] = false;
     }
 
     basedir = (string) getenv("HOME") + "/.centericq/";
@@ -82,7 +81,8 @@ icqconf::imserver icqconf::defservers[protocolname_size] = {
     { "irc.oftc.net", 6667, 0 },
     { "jabber.com", 5222, 5223 },
     { "", 0, 0 },
-    { "livejournal.com", 80, 0 }
+    { "livejournal.com", 80, 0 },
+    { "appmsg.gadu-gadu.pl", 80 }
 };
 
 void icqconf::setourid(const imaccount &im) {
@@ -112,6 +112,16 @@ void icqconf::setourid(const imaccount &im) {
 	    i->port = defservers[i->pname].port;
 
 	switch(i->pname) {
+	    case jabber:
+		if(i->additional.find("prio") == i->additional.end())
+		    i->additional["prio"] = "4";
+		break;
+
+	    case livejournal:
+		if(i->additional.find("importfriends") == i->additional.end())
+		    i->additional["importfriends"] = "1";
+		break;
+
 	    case icq:
 		if(i->password.size() > 8)
 		    i->password = i->password.substr(0, 8);
@@ -180,11 +190,188 @@ void icqconf::checkdir() {
 
 void icqconf::load() {
     loadmainconfig();
+    loadkeys();
     loadcolors();
     loadactions();
     external.load();
 }
 
+vector<icqconf::keybinding> icqconf::keys;
+
+void icqconf::loadkeys() {
+    int x;
+    string fname = getconfigfname("keybindings"), buf, param;
+    struct keybinding k;
+    ifstream f;
+
+    keys.clear();
+
+    if(access(fname.c_str(), F_OK)) {
+	ofstream of(fname.c_str());
+
+	if(of.is_open()) {
+	    of << "# This file contains keybinding configuration for centericq" << endl;
+	    of << "# Every line should look like: bind <section> <key> <command>" << endl << endl;
+	    of << "bind contact\t?\tinfo" << endl;
+	    of << "bind contact\tq\tquit" << endl;
+	    of << "bind contact\t<F3>\tchange_status" << endl;
+	    of << "bind contact\ts\tchange_status" << endl;
+	    of << "bind contact\th\thistory" << endl;
+	    of << "bind contact\tf\tfetch_away_message" << endl;
+	    of << "bind contact\t<F2>\tuser_menu" << endl;
+	    of << "bind contact\tm\tuser_menu" << endl;
+	    of << "bind contact\t<F4>\tgeneral_menu" << endl;
+	    of << "bind contact\tg\tgeneral_menu" << endl;
+	    of << "bind contact\t<F5>\thide_offline" << endl;
+	    of << "bind contact\t<F6>\tcontact_external_action" << endl;
+	    of << "bind contact\ta\tadd" << endl;
+	    of << "bind contact\t<delete> remove" << endl;
+	    of << "bind contact\t<enter> compose" << endl;
+	    of << "bind contact\tc\tsend_contact" << endl;
+	    of << "bind contact\tc\trss_check" << endl;
+	    of << "bind contact\tu\tsend_url" << endl;
+	    of << "bind contact\tr\trename" << endl;
+	    of << "bind contact\tv\tversion" << endl;
+	    of << "bind contact\te\tedit" << endl;
+	    of << "bind contact\ti\tignore" << endl;
+	    of << "bind contact\t\\as\tquickfind" << endl;
+	    of << "bind contact\t/\tquickfind" << endl << endl;
+	    of << "bind history\t/\tsearch" << endl;
+	    of << "bind history\ts\tsearch" << endl;
+	    of << "bind history\tn\tsearch_again" << endl;
+	    of << "bind history\t<F2>\tshow_urls" << endl;
+	    of << "bind history\t<F9>\tfullscreen" << endl << endl;
+	    of << "bind editor\t\\cx\tsend_message" << endl;
+	    of << "bind editor\t<esc>\tquit" << endl;
+	    of << "bind editor\t\\cp\tmultiple_recipients" << endl;
+	    of << "bind editor\t\\co\thistory" << endl;
+	    of << "bind editor\t\\a?\tinfo" << endl;
+	    of << "bind editor\t<F2>\tshow_urls" << endl;
+	    of << "bind editor\t<F9>\tfullscreen" << endl << endl;
+	    of << "bind info\t<F2>\tshow_urls" << endl;
+	    of << "bind info\t<F6>\tuser_external_action" << endl;
+
+	    of.close();
+	}
+    }
+
+    f.open(fname.c_str());
+
+    if(f.is_open()) {
+
+	while(!f.eof()) {
+	    getstring(f, buf);
+
+	    if(buf.empty() || buf[0] == '#') continue;
+
+	    k.ctrl = false;
+	    k.alt = false;
+
+	    param = getword(buf);	/* first word is always bind */
+
+	    param = getword(buf);	/* read section */
+	    if(param == "contact")
+		k.section = section_contact; else
+	    if(param == "history")
+		k.section = section_history; else
+	    if (param == "editor")
+		k.section = section_editor; else
+	    if (param == "info")
+		k.section = section_info;
+
+	    param = getword(buf);	/* key */
+	    if(param == "\\c<space>" || param == "\\C<space>" || param == "^<space>") {
+		k.ctrl = true;
+		k.key = CTRL(' ');
+	    } else if(param == "\\a<enter>" || param == "\\A<enter>" || param == "@<enter>") {
+		k.alt = true;
+		k.key = ALT('\r');
+	    } else if(param == "\\a<space>" || param == "\\A<space>" || param == "@<space>") {
+		k.ctrl = true;
+		k.key = ALT(' ');
+	    } else if(param[0] == '\\') {
+		if(param[1] == 'c' || param[1] == 'C') {
+		    k.ctrl = true;
+		    k.key = CTRL(param[2]);
+		} else if(param[1] == 'a' || param[1] == 'A') {
+		    k.alt = true;
+		    k.key = ALT(param[2]);
+		} else if(param[1] == '\\') {
+		    k.key = '\\';
+		} else if(param[1] == '@') {
+		    k.key = '@';
+		} else if(param[1] == '^') {
+		    k.key = '^';
+		}
+	    } else if(param[0] == '^') {
+		k.ctrl = true;
+		k.key = CTRL(param[1]);
+	    } else if(param[0] == '@') {
+		k.alt = true;
+		k.key = ALT(param[1]);
+	    } else if(param == "<delete>")
+		k.key = KEY_DC;
+	    else if(param == "<enter>")
+		k.key = '\r';
+	    else if(param == "<esc>")
+		k.key = 27;
+	    else if(param == "<space>")
+		k.key = ' ';
+	    else if(param == "<backspace>")
+		k.key = '\b';
+	    else if(param == "<pageup>")
+		k.key = 339;
+	    else if(param == "<pagedown>")
+		k.key = 338;
+	    else if(param == "<tab>")
+		k.key = 9;
+	    else if(param == "<insert>")
+		k.key = 331;
+	    else if(param[0] == '<' && param[1] == 'F') {
+		k.alt = true;
+		k.ctrl = true;
+		k.key = KEY_F(param[2]-'0');
+	    } else
+		k.key = param[0];
+
+	    param = getword(buf);	/* action */
+	    if(param == "info") k.action = key_info; else
+	    if(param == "remove") k.action = key_remove; else
+	    if(param == "quit") k.action = key_quit; else
+	    if(param == "change_status") k.action = key_change_status; else
+	    if(param == "history") k.action = key_history; else
+	    if(param == "fetch_away_message") k.action = key_fetch_away_message; else
+	    if(param == "user_menu") k.action = key_user_menu; else
+	    if(param == "general_menu") k.action = key_general_menu; else
+	    if(param == "hide_offline") k.action = key_hide_offline; else
+	    if(param == "contact_external_action") k.action = key_contact_external_action; else
+	    if(param == "add") k.action = key_add; else
+	    if(param == "remove") k.action = key_remove; else
+	    if(param == "send_message") k.action = key_send_message; else
+	    if(param == "send_contact") k.action = key_send_contact; else
+	    if(param == "send_url") k.action = key_send_url; else
+	    if(param == "rename") k.action = key_rename; else
+	    if(param == "version") k.action = key_version; else
+	    if(param == "fullscreen") k.action = key_fullscreen; else
+	    if(param == "edit") k.action = key_edit; else
+	    if(param == "ignore") k.action = key_ignore; else
+	    if(param == "quickfind") k.action = key_quickfind; else
+	    if(param == "search") k.action = key_search; else
+	    if(param == "compose") k.action = key_compose; else
+	    if(param == "search_again") k.action = key_search_again; else
+	    if(param == "show_urls") k.action = key_show_urls; else
+	    if(param == "rss_check") k.action = key_rss_check; else
+	    if(param == "multiple_recipients") k.action = key_multiple_recipients; else
+	    if(param == "user_external_action") k.action = key_user_external_action;
+	    else continue;
+
+	    keys.push_back(k);
+	}
+	f.close();
+    }
+}
+
+
 void icqconf::loadmainconfig() {
     string fname = getconfigfname("config"), buf, param, rbuf;
     ifstream f(fname.c_str());
@@ -196,7 +383,6 @@ void icqconf::loadmainconfig() {
 	savepwd = bidi = true;
 	setsmtphost("");
 	setpeertopeer(0, 65535);
-	defcharset = "ISO-8859-1";
 
 	while(!f.eof()) {
 	    getstring(f, buf);
@@ -220,12 +406,14 @@ void icqconf::loadmainconfig() {
 	    if(param == "log") makelog = true; else
 	    if(param == "chatmode") initmultiproto(chatmode, buf); else
 	    if(param == "entersends") initmultiproto(entersends, buf); else
-	    if(param == "russian") initmultiproto(russian, buf); else
+	    if(param == "nonimonline") initmultiproto(nonimonline, buf); else
+	    if(param == "russian" || param == "convert") initmultiproto(cpconvert, buf); else
 	    if(param == "nobidi") setbidi(false); else
 	    if(param == "askaway") askaway = true; else
 	    if(param == "logtimestamps") logtimestamps = true; else
-	    if(param == "defcharset") setdefcharset(buf); else
 	    if(param == "logonline") logonline = true; else
+	    if(param == "fromcharset") fromcharset = buf; else
+	    if(param == "tocharset") tocharset = buf; else
 	    if(param == "ptp") {
 		ptpmin = atoi(getword(buf, "-").c_str());
 		ptpmax = atoi(buf.c_str());
@@ -241,6 +429,15 @@ void icqconf::loadmainconfig() {
 	    }
 	}
 
+	if(fromcharset.empty() && tocharset.empty())
+	for(pname = icq; pname != protocolname_size; (int) pname += 1) {
+	    if(getcpconvert(pname)) {
+		fromcharset = "cp1251";
+		tocharset = "koi8-r";
+		break;
+	    }
+	}
+
 	f.close();
     }
 }
@@ -271,7 +468,6 @@ void icqconf::save() {
 	    if(getantispam()) f << "antispam" << endl;
 	    if(getmailcheck()) f << "mailcheck" << endl;
 	    if(getaskaway()) f << "askaway" << endl;
-	    f << "defcharset\t" << getdefcharset() << endl;
 
 	    param = "";
 	    for(protocolname pname = icq; pname != protocolname_size; (int) pname += 1)
@@ -287,9 +483,18 @@ void icqconf::save() {
 
 	    param = "";
 	    for(protocolname pname = icq; pname != protocolname_size; (int) pname += 1)
-		if(getrussian(pname)) param += (string) " " + conf.getprotocolname(pname);
+		if(getnonimonline(pname)) param += (string) " " + conf.getprotocolname(pname);
 	    if(!param.empty())
-		f << "russian" << param << endl;
+		f << "nonimonline" << param << endl;
+
+	    param = "";
+	    for(protocolname pname = icq; pname != protocolname_size; (int) pname += 1)
+		if(getcpconvert(pname)) param += (string) " " + conf.getprotocolname(pname);
+	    if(!param.empty())
+		f << "convert" << param << endl;
+
+	    f << "fromcharset\t" << fromcharset << endl;
+	    f << "tocharset\t" << tocharset << endl;
 
 	    if(!getbidi()) f << "nobidi" << endl;
 	    if(logtimestamps) f << "logtimestamps" << endl;
@@ -298,7 +503,10 @@ void icqconf::save() {
 	    f << "smtp\t" << getsmtphost() << ":" << getsmtpport() << endl;
 
 	    if(!gethttpproxyhost().empty())
-		f << "http_proxy\t" << gethttpproxyhost() << ":" << gethttpproxyport() << endl;
+		if (!gethttpproxyuser().empty())
+		    f << "http_proxy\t" << gethttpproxyuser() << ":" << gethttpproxypasswd() << "@" << gethttpproxyhost() << ":" << gethttpproxyport() << endl;
+		else
+		    f << "http_proxy\t" << gethttpproxyhost() << ":" << gethttpproxyport() << endl;
 
 	    f << "ptp\t" << ptpmin << "-" << ptpmax << endl;
 
@@ -334,8 +542,8 @@ void icqconf::loadcolors() {
 	    schemer.push(cp_main_selected, "main_selected   black/white");
 	    schemer.push(cp_main_highlight, "main_highlight yellow/transparent  bold");
 	    schemer.push(cp_main_frame, "main_frame blue/transparent    bold");
-	    schemer.push(cp_main_history_incoming, "main_history_incoming  cyan/transparent");
-	    schemer.push(cp_main_history_outgoing, "main_history_outgoing  yellow/transparent  bold");
+	    schemer.push(cp_main_history_incoming, "main_history_incoming  yellow/transparent    bold");
+	    schemer.push(cp_main_history_outgoing, "main_history_outgoing  cyan/transparent");
 	    schemer.push(cp_clist_icq, "clist_icq   green/transparent");
 	    schemer.push(cp_clist_yahoo, "clist_yahoo   magenta/transparent");
 	    schemer.push(cp_clist_infocard, "clist_infocard white/transparent");
@@ -345,6 +553,7 @@ void icqconf::loadcolors() {
 	    schemer.push(cp_clist_jabber, "clist_jabber    red/transparent");
 	    schemer.push(cp_clist_rss, "clist_rss    white/transparent   bold");
 	    schemer.push(cp_clist_lj, "clist_lj    cyan/transparent   bold");
+	    schemer.push(cp_clist_gadu, "clist_gg    blue/transparent   bold");
 	    break;
 
 	case rcblue:
@@ -370,6 +579,7 @@ void icqconf::loadcolors() {
 	    schemer.push(cp_clist_jabber, "clist_jabber    red/blue");
 	    schemer.push(cp_clist_rss, "clist_rss    white/blue   bold");
 	    schemer.push(cp_clist_lj, "clist_lj    cyan/blue   bold");
+	    schemer.push(cp_clist_gadu, "clist_gg    blue/blue   bold");
 	    break;
     }
 
@@ -480,10 +690,18 @@ void icqconf::loadsounds() {
 		    for(pname = icq; pname != protocolname_size && skey != getprotocolname(pname); (int) pname += 1);
 
 		    if(pname != protocolname_size) {
-			if(pname == icq) ic = imcontact(strtoul(suin.c_str(), 0, 0), pname);
-			else ic = imcontact(suin, pname);
+			if(suin == "*") {
+			    for(i = 0; i < clist.count; i++) {
+				c = (icqcontact *) clist.at(i);
 
-			c = clist.get(ic);
+				if(c->getdesc().pname == pname) c->setsound(it, buf);
+			    }
+			} else {
+			    if(pname == icq) ic = imcontact(strtoul(suin.c_str(), 0, 0), pname);
+				else ic = imcontact(suin, pname);
+
+			    c = clist.get(ic);
+			}
 		    }
 		}
 	    } else {
@@ -670,12 +888,20 @@ void icqconf::setentersends(protocolname pname, bool fentersends) {
     entersends[pname] = fentersends;
 }
 
-bool icqconf::getrussian(protocolname pname) {
-    return russian[pname];
+bool icqconf::getnonimonline(protocolname pname) {
+    return nonimonline[pname];
 }
 
-void icqconf::setrussian(protocolname pname, bool frussian) {
-    russian[pname] = frussian;
+void icqconf::setnonimonline(protocolname pname, bool fnonimonline) {
+    nonimonline[pname] = fnonimonline;
+}
+
+bool icqconf::getcpconvert(protocolname pname) const {
+    return cpconvert[pname];
+}
+
+void icqconf::setcpconvert(protocolname pname, bool fcpconvert) {
+    cpconvert[pname] = fcpconvert;
 }
 
 string icqconf::execaction(const string &name, const string &param) {
@@ -736,7 +962,7 @@ string icqconf::execaction(const string &name, const string &param) {
 
 string icqconf::getprotocolname(protocolname pname) const {
     static const string ptextnames[protocolname_size] = {
-	"icq", "yahoo", "msn", "aim", "irc", "jabber", "rss", "lj", "infocard"
+	"icq", "yahoo", "msn", "aim", "irc", "jab", "rss", "lj", "gg", "infocard"
     };
 
     return ptextnames[pname];
@@ -752,6 +978,7 @@ protocolname icqconf::getprotocolbyletter(char letter) const {
 	case 'j': return jabber;
 	case 'r': return rss;
 	case 'l': return livejournal;
+	case 'g': return gadu;
 	case '0':
 	case '1':
 	case '2':
@@ -769,7 +996,7 @@ protocolname icqconf::getprotocolbyletter(char letter) const {
 
 string icqconf::getprotocolprefix(protocolname pname) const {
     static const string pprefixes[protocolname_size] = {
-	"", "y", "m", "a", "i", "j", "r", "l", "n"
+	"", "y", "m", "a", "i", "j", "r", "l", "g", "n"
     };
 
     return pprefixes[pname];
@@ -807,6 +1034,7 @@ int icqconf::getprotcolor(protocolname pname) const {
 	case jabber : return getcolor(cp_clist_jabber);
 	case rss : return getcolor(cp_clist_rss);
 	case livejournal : return getcolor(cp_clist_lj);
+	case gadu : return getcolor(cp_clist_gadu);
 	default : return getcolor(cp_main_text);
     }
 }
@@ -868,7 +1096,13 @@ void icqconf::commandline(int argc, char **argv) {
 
 	} else if((args == "-v") || (args == "--version")) {
 	    cout << PACKAGE << " " << VERSION << endl
-		<< "Written by Konstantin Klyagin." << endl << endl
+		<< "Written by Konstantin Klyagin." << endl
+		<< "Built-in protocols are:";
+
+	    for(protocolname pname = icq; pname != protocolname_size; (int) pname += 1)
+		if(gethook(pname).enabled()) cout << " " << conf.getprotocolname(pname);
+
+	    cout << endl << endl
 		<< "This is free software; see the source for copying conditions.  There is NO" << endl
 		<< "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE." << endl;
 
@@ -1091,15 +1325,26 @@ void icqconf::setsmtphost(const string &asmtphost) {
 }
 
 void icqconf::sethttpproxyhost(const string &ahttpproxyhost) {
-    int pos;
+    int pos, posn;
 
     httpproxyhost = "";
     httpproxyport = 0;
+    httpproxyuser = "";
+    httpproxypasswd = "";
 
     if(!ahttpproxyhost.empty()) {
 	if((pos = ahttpproxyhost.find(":")) != -1) {
-	    httpproxyhost = ahttpproxyhost.substr(0, pos);
-	    httpproxyport = atol(ahttpproxyhost.substr(pos+1).c_str());
+	    if ((posn = ahttpproxyhost.find("@")) != -1) {
+		httpproxyuser = ahttpproxyhost.substr(0, pos);
+		httpproxypasswd = ahttpproxyhost.substr(pos+1, posn-pos-1);
+		if((pos = ahttpproxyhost.find(":", posn)) != -1) {
+		    httpproxyhost = ahttpproxyhost.substr(posn+1, pos-posn-1);
+		    httpproxyport = atol(ahttpproxyhost.substr(pos+1).c_str());
+		}
+	    } else {
+		httpproxyhost = ahttpproxyhost.substr(0, pos);
+		httpproxyport = atol(ahttpproxyhost.substr(pos+1).c_str());
+	    }
 	} else {
 	    httpproxyhost = ahttpproxyhost;
 	    httpproxyport = 8080;
@@ -1123,6 +1368,14 @@ unsigned int icqconf::gethttpproxyport() const {
     return httpproxyport ? httpproxyport : 8080;
 }
 
+string icqconf::gethttpproxyuser() const {
+    return httpproxyuser;
+}
+
+string icqconf::gethttpproxypasswd() const {
+    return httpproxypasswd;
+}
+
 void icqconf::checkdiskspace() {
     fenoughdiskspace = true;
 
@@ -1137,6 +1390,21 @@ void icqconf::checkdiskspace() {
     }
 }
 
+void icqconf::setcharsets(const string &from, const string &to) {
+    fromcharset = from;
+    tocharset = to;
+}
+
+const char *icqconf::getconvertfrom(protocolname pname) const {
+    if(pname != protocolname_size && !getcpconvert(pname)) return "iso-8859-1";
+    return fromcharset.c_str();
+}
+
+const char *icqconf::getconvertto(protocolname pname) const {
+    if(pname != protocolname_size && !getcpconvert(pname)) return "iso-8859-1";
+    return tocharset.c_str();
+}
+
 // ----------------------------------------------------------------------------
 
 icqconf::imaccount::imaccount() {
