@@ -39,7 +39,42 @@ enum cicq_colorpairs {
     cp_clist_irc,
     cp_clist_jabber,
     cp_clist_rss,
-    cp_clist_lj
+    cp_clist_lj,
+    cp_clist_gadu
+};
+
+enum cicq_keybindings {
+    section_contact,
+    section_history,
+    section_editor,
+    section_info,
+    key_info,
+    key_quit,
+    key_change_status,
+    key_history,
+    key_fetch_away_message,
+    key_user_menu,
+    key_general_menu,
+    key_hide_offline,
+    key_contact_external_action,
+    key_add,
+    key_fullscreen,
+    key_remove,
+    key_send_message,
+    key_compose,
+    key_send_contact,
+    key_send_url,
+    key_rename,
+    key_version,
+    key_edit,
+    key_ignore,
+    key_quickfind,
+    key_search,
+    key_search_again,
+    key_show_urls,
+    key_rss_check,
+    key_multiple_recipients,
+    key_user_external_action
 };
 
 class icqconf {
@@ -53,6 +88,11 @@ class icqconf {
 	    int port, secureport;
 	};
 
+	struct keybinding {
+	    int section, key, action;
+	    bool alt, ctrl;
+	};
+
 	struct imaccount {
 	    imaccount();
 	    imaccount(protocolname apname);
@@ -73,6 +113,8 @@ class icqconf {
 
 	static imserver defservers[];
 
+	static vector<keybinding> keys;
+
     protected:
 	vector<icqgroup> groups;
 	vector<imaccount> accounts;
@@ -85,11 +127,14 @@ class icqconf {
 	    makelog, fenoughdiskspace, askaway, bidi, logtimestamps,
 	    logonline, emacs;
 
-	bool chatmode[protocolname_size], russian[protocolname_size],
-	    entersends[protocolname_size];
+	bool chatmode[protocolname_size];
+	bool cpconvert[protocolname_size];
+	bool entersends[protocolname_size];
+	bool nonimonline[protocolname_size];
 
 	string sockshost, socksuser, sockspass, basedir, argv0, smtphost,
-	    bindhost, httpproxyhost, defcharset;
+	    bindhost, httpproxyhost, httpproxyuser, httpproxypasswd,
+	    fromcharset, tocharset;
 
 	map<string, string> actions;
 
@@ -121,6 +166,7 @@ class icqconf {
 
 	void save();
 
+	void loadkeys();
 	void loadcolors();
 	void loadsounds();
 	void loadactions();
@@ -155,8 +201,8 @@ class icqconf {
 	bool getmakelog() const { return makelog; }
 	void setmakelog(bool slog);
 
-	bool getrussian(protocolname pname);
-	void setrussian(protocolname pname, bool frussian);
+	bool getcpconvert(protocolname pname) const;
+	void setcpconvert(protocolname pname, bool fcpconvert);
 
 	bool getbidi() const { return bidi; }
 	void setbidi(bool fbidi);
@@ -170,6 +216,9 @@ class icqconf {
 	bool getentersends(protocolname pname);
 	void setentersends(protocolname pname, bool fentersends);
 
+	bool getnonimonline(protocolname pname);
+	void setnonimonline(protocolname pname, bool fnonimonline);
+
 	groupmode getgroupmode() const { return fgroupmode; }
 	void setgroupmode(groupmode amode);
 
@@ -184,6 +233,8 @@ class icqconf {
 	string gethttpproxyhost() const;
 	unsigned int gethttpproxyport() const;
 	void sethttpproxyhost(const string &ahttpproxyhost);
+	string gethttpproxyuser() const;
+	string gethttpproxypasswd() const;
 
 	void getsocksuser(string &name, string &pass) const;
 	void setsocksuser(const string &name, const string &pass);
@@ -223,8 +274,10 @@ class icqconf {
 	void setlogoptions(bool flogtimestamps, bool flogonline)
 	    { logtimestamps = flogtimestamps, logonline = flogonline; }
 
-	const char *getdefcharset() const { return defcharset.c_str(); }
-	void setdefcharset(const string &adefcharset) { defcharset = adefcharset; }
+	void setcharsets(const string &from, const string &to);
+
+	const char *getconvertfrom(protocolname pname = protocolname_size) const;
+	const char *getconvertto(protocolname pname = protocolname_size) const;
 };
 
 extern icqconf conf;
