@@ -1,9 +1,9 @@
 /*
 *
 * centericq contact list class
-* $Id: icqcontacts.cc,v 1.49 2003/11/22 23:58:55 konst Exp $
+* $Id: icqcontacts.cc,v 1.52 2004/03/07 13:44:41 konst Exp $
 *
-* Copyright (C) 2001,2002 by Konstantin Klyagin <konst@konst.org.ua>
+* Copyright (C) 2001-2004 by Konstantin Klyagin <konst@konst.org.ua>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -34,11 +34,12 @@ icqcontacts::icqcontacts() {
 icqcontacts::~icqcontacts() {
 }
 
-icqcontact *icqcontacts::addnew(const imcontact &cinfo, bool notinlist) {
+icqcontact *icqcontacts::addnew(const imcontact &cinfo, bool notinlist, int agroupid, bool reqauth) {
     icqcontact *c = new icqcontact(cinfo);
 
     switch(cinfo.pname) {
 	case icq:
+	case gadu:
 	    c->setnick(i2str(cinfo.uin));
 	    c->setdispnick(i2str(cinfo.uin));
 	    break;
@@ -49,6 +50,14 @@ icqcontact *icqcontacts::addnew(const imcontact &cinfo, bool notinlist) {
 	    break;
     }
 
+    icqcontact::basicinfo bi = c->getbasicinfo();
+    bi.requiresauth = bi.authawait = reqauth;
+    c->setbasicinfo(bi);
+
+    if(agroupid) {
+	c->setgroupid(agroupid, false);
+    }
+
     c->save();
     add(c);
 
@@ -97,6 +106,7 @@ void icqcontacts::load() {
 
 		    case infocard:
 		    case rss:
+		    case gadu:
 			c = new icqcontact(imcontact(atol(ent->d_name+1), pname));
 			break;
 
@@ -105,6 +115,21 @@ void icqcontacts::load() {
 			// supported protocols.
 			break;
 
+		    case jabber:
+			if(get(imcontact(ent->d_name+1, pname)))
+			    continue;
+
+			if(string(ent->d_name+1).find("@") == -1) {
+			    string oldname = conf.getdirname() + ent->d_name;
+			    string ndname = oldname + "@jabber.com";
+
+			    if(!access(ndname.c_str(), F_OK))
+				rename(ndname.c_str(), (conf.getdirname() + "_" + ent->d_name).c_str());
+
+			    rename(oldname.c_str(), ndname.c_str());
+			    strcpy(ent->d_name, justfname(ndname).c_str());
+			}
+
 		    default:
 			c = new icqcontact(imcontact(ent->d_name+1, pname));
 			c->setnick(ent->d_name+1);
