@@ -1,7 +1,7 @@
 /*
 *
 * centericq user interface class, dialogs related part
-* $Id: icqdialogs.cc,v 1.141 2004/03/09 21:46:26 konst Exp $
+* $Id: icqdialogs.cc,v 1.143 2004/04/01 08:01:13 konst Exp $
 *
 * Copyright (C) 2001-2004 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -32,7 +32,7 @@
 #include "abstracthook.h"
 #include "ljhook.h"
 
-#include <libicq2000/userinfoconstants.h>
+#include <libicq2000/constants.h>
 
 const char *stragerange(ICQ2000::AgeRange r) {
     switch(r) {
@@ -661,6 +661,13 @@ void icqface::gendetails(treeview *tree, icqcontact *c) {
 		tree->addleaff(i, 0, 44, _(" Authorization required : %s "), stryesno(bi.requiresauth));
 	    }
 	}
+    } else if(passinfo.pname == gadu) {
+	CHECKGENERAL;
+
+	tree->addleaff(i, 0, 11, _(" First name : %s "), bi.fname.c_str());
+	tree->addleaff(i, 0, 12, _(" Last name : %s "), bi.lname.c_str());
+	tree->addleaff(i, 0, 17, _(" City : %s "), bi.city.c_str());
+	tree->addleaff(i, 0, 14, _(" Gender : %s "), strgender(mi.gender));
     }
 
     if(passinfo.pname == rss) {
@@ -1429,7 +1436,8 @@ void icqface::selectproto(bool prots[], spmode mode) {
 		continue;
 
 	} else {
-	    if(mode != spIMplusRSS || pname != rss) {
+	    if(mode != spIMplusRSS || pname != rss)
+	    if(pname != livejournal || mode == spIMonly) {
 		if(gethook(pname).getCapabs().count(hookcapab::nochat))
 		    continue;
 
