@@ -1,7 +1,7 @@
 /*
 *
 * centericq user interface class
-* $Id: icqface.cc,v 1.218 2004/03/20 16:49:08 konst Exp $
+* $Id: icqface.cc,v 1.222 2004/04/11 16:32:28 konst Exp $
 *
 * Copyright (C) 2001-2004 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -211,10 +211,10 @@ void icqface::showtopbar() {
 	ia = conf.getourid(pname);
 
 	if(!ia.empty()) {
-	    buf += conf.getprotocolname(pname) + "<";
-
+	    buf += " ";
+	    buf += conf.getprotocolname(pname) + ":";
 	    buf += imstatus2char[gethook(pname).getstatus()];
-	    buf += ">";
+//            buf += ">";
 	}
     }
 
@@ -1718,14 +1718,17 @@ void icqface::modelist(contactstatus cs) {
 bool icqface::multicontacts(const string &ahead,
 const set<protocolname> &protos, contactstatus cs) {
 
-    int i, savefirst, saveelem;
+    int i, savefirst, saveelem, prevgid;
     bool ret = true, finished = false;
     string head = ahead;
 
     imcontact ic;
-    vector<imcontact>::iterator c;
+    vector<imcontact>::iterator c, cc;
     vector<imcontact> mlst;
 
+    map<int, bool> groupcheck;
+    icqcontact *cont;
+
     verticalmenu m(sizeWArea.x1+1, sizeWArea.y1+3, sizeWArea.x2, sizeWArea.y2,
 	conf.getcolor(cp_main_text), conf.getcolor(cp_main_selected));
 
@@ -1737,14 +1740,21 @@ const set<protocolname> &protos, contactstatus cs) {
     if(!head.size()) head = _("Event recipients");
     mainw.write(sizeWArea.x1+2, sizeWArea.y1, conf.getcolor(cp_main_highlight), head);
 
-    for(i = 0; i < clist.count; i++) {
-	icqcontact *c = (icqcontact *) clist.at(i);
-	imcontact desc = c->getdesc();
+    vector<icqgroup>::iterator ig = groups.begin();
 
-	if(!desc.empty())
-	if(protos.empty() || protos.count(desc.pname))
-	if(!lst.inlist(desc, cs))
-	    mlst.push_back(desc);
+    while(ig != groups.end()) {
+	for(i = 0; i < clist.count; i++) {
+	    icqcontact *c = (icqcontact *) clist.at(i);
+	    imcontact desc = c->getdesc();
+
+	    if(!desc.empty())
+	    if(c->getgroupid() == ig->getid())
+	    if(protos.empty() || protos.count(desc.pname))
+	    if(!lst.inlist(desc, cs))
+		mlst.push_back(desc);
+	}
+
+	ig++;
     }
 
     m.idle = &menuidle;
@@ -1754,13 +1764,26 @@ const set<protocolname> &protos, contactstatus cs) {
 	m.getpos(saveelem, savefirst);
 	m.clear();
 
+	int prevgid = -1;
+
 	for(c = mlst.begin(); c != mlst.end(); ++c) {
-	    icqcontact *cont = (icqcontact *) clist.get(*c);
+	    cont = (icqcontact *) clist.get(*c);
+
+	    if(conf.getgroupmode() != icqconf::nogroups)
+	    if(cont->getgroupid() != prevgid) {
+		prevgid = cont->getgroupid();
+
+		if(groupcheck.find(prevgid) == groupcheck.end())
+		    groupcheck[prevgid] = false;
+
+		m.additemf(conf.getcolor(cp_main_highlight), 0, "\002[%c]\002\002 %s",
+		    groupcheck[prevgid] ? 'x' : ' ',
+		    groups.getname(prevgid).c_str());
+	    }
 
 	    m.additemf(conf.getprotcolor(c->pname), cont, " [%c] %s",
 		find(muins.begin(), muins.end(), *c) != muins.end() ? 'x' : ' ',
-		cont->getdispnick().c_str()
-	    );
+		cont->getdispnick().c_str());
 	}
 
 	m.setpos(saveelem, savefirst);
@@ -1771,10 +1794,30 @@ const set<protocolname> &protos, contactstatus cs) {
 		    ic = ((icqcontact *) m.getref(m.getpos()))->getdesc();
 		    c = find(muins.begin(), muins.end(), ic);
 
-		    if(c != muins.end()) {
-			muins.erase(c);
-		    } else {
-			muins.push_back(ic);
+		    if(c != muins.end()) muins.erase(c);
+			else muins.push_back(ic);
+
+		    cont = clist.get(ic);
+		    groupcheck[cont->getgroupid()] = false;
+
+		} else {
+		    bool been = false;
+		    int gid = ((icqcontact *) m.getref(m.getpos()+1))->getgroupid();
+		    groupcheck[gid] = !groupcheck[gid];
+
+		    for(c = mlst.begin(); c != mlst.end(); ++c) {
+			cont = (icqcontact *) clist.get(*c);
+
+			if(cont->getgroupid() == gid) {
+			    been = true;
+			    cc = find(muins.begin(), muins.end(), *c);
+
+			    if(groupcheck[gid] && cc == muins.end()) {
+				muins.push_back(*c);
+			    } else if(!groupcheck[gid] && cc != muins.end()) {
+				muins.erase(cc);
+			    }
+			} else if(been) break;
 		    }
 		}
 		break;
@@ -2659,8 +2702,8 @@ vector<eventviewresult> abuttons, bool nobuttons) {
 	extracturls(text);
 
 	status(_("%s to URLs, %s to full-screenize, %s close"),
-	    getstatkey(key_show_urls, section_info).c_str(),
-	    getstatkey(key_fullscreen, section_editor).c_str(),
+	    getstatkey(key_show_urls, section_history).c_str(),
+	    getstatkey(key_fullscreen, section_history).c_str(),
 	    getstatkey(key_quit, section_editor).c_str());
     }
 
@@ -2829,6 +2872,10 @@ bool icqface::histexec(imevent *&im) {
 			    }
 			}
 			break;
+		    case -4:
+			r = false;
+			fin = true;
+			break;
 
 		    default:
 			if(mhist.getref(k-1)) {
@@ -3137,6 +3184,8 @@ int icqface::historykeys(dialogbox &db, int k) {
 	    return -2;
 	case key_search_again:
 	    return -3;
+	case key_quit:
+	    return -4;
     }
 
     return -1;
@@ -3145,10 +3194,14 @@ int icqface::historykeys(dialogbox &db, int k) {
 int icqface::editmsgkeys(texteditor &e, int k) {
     char *p;
 
+    if(k == '\r' && conf.getentersends(face.passinfo.pname)) {
+	p = e.save("");
+	face.editdone = strlen(p);
+	delete p;
+	if(face.editdone) return -1; else return 0;
+    }
+
     switch(face.key2action(k, section_editor)) {
-	case '\r':
-	    if(!conf.getentersends(face.passinfo.pname))
-		break;
 	case key_send_message:
 	    p = e.save("");
 	    face.editdone = strlen(p);
@@ -3193,13 +3246,15 @@ int icqface::eventviewkeys(dialogbox &db, int k) {
 	case key_fullscreen:
 	    face.fullscreenize(face.passevent);
 	    break;
+	case key_quit:
+	    return 0;
     }
 
     return -1;
 }
 
 int icqface::findreskeys(dialogbox &db, int k) {
-    if (face.key2action(k, section_history) == key_search)
+    if(face.key2action(k, section_contact) == key_quickfind)
 	return -3;
 
     return -1;
