@@ -25,7 +25,8 @@ enum interfaceAction {
     ACT_PING, ACT_EDITUSER, ACT_VISIBLELIST, ACT_INVISLIST, ACT_QUICKFIND,
     ACT_FILE, ACT_GROUPMOVE, ACT_ORG_GROUPS, ACT_HIDEOFFLINE, ACT_FETCHAWAY,
     ACT_EMAIL, ACT_AUTH, ACT_CONTACT, ACT_VERSION, ACT_JOIN, ACT_LEAVE,
-    ACT_CONFER, ACT_TRANSFERS, ACT_JOINDIALOG, ACT_EXTERN, ACT_RSS, ACT_LJ
+    ACT_CONFER, ACT_TRANSFERS, ACT_JOINDIALOG, ACT_EXTERN, ACT_RSS, ACT_LJ,
+    ACT_MASS_MOVE
 };
 
 extern class centericq cicq;
@@ -114,6 +115,7 @@ class icqface {
 	static int findreskeys(dialogbox &db, int k);
 	static int eventviewkeys(dialogbox &db, int k);
 	static int statuskeys(verticalmenu &m, int k);
+	static int fullscreenkeys(textbrowser &m, int k);
 
 	static void editidle(texteditor &e);
 	static void textbrowseridle(textbrowser &b);
@@ -144,7 +146,9 @@ class icqface {
 	void selectlanguage(unsigned short &f);
 	void selectagerange(ICQ2000::AgeRange &r);
 	void selectrandomgroup(unsigned short &f);
-	void selectproto(bool prots[], bool irss = false);
+
+	enum spmode { spIMonly, spIMplusRSS, spnonIM };
+	void selectproto(bool prots[], spmode mode = spIMonly);
 
 	void workarealine(int l, chtype c = HLINE);
 
@@ -157,8 +161,14 @@ class icqface {
 	bool sprofmanager(string &name, string &act);
 	void showeventbottom(const imcontact &ic);
 
+	int key2action(int k, int s);
+	string action2key(int a, int s, int n = 1) const;
+
 	void invokelist(string &s, vector<string> &v, const string &def, textwindow *w);
 	bool setljparams(imxmlevent *ev);
+	void multichange(bool conv[], bool newstate);
+
+	string extracttime(const imevent &ev);
 
     public:
 	int extk;
@@ -173,6 +183,9 @@ class icqface {
 	static void menuidle(verticalmenu &caller);
 	static void dialogidle(dialogbox &caller);
 
+	string getmenuitem(string mtext, int width, int key, int section);
+	string getstatkey(int key, int section) const;
+
 	void redraw();
 
 	void draw();
@@ -193,6 +206,7 @@ class icqface {
 	void log(const char *fmt, ...);
 
 	void status(const string &text);
+	void status(const char *fmt, ...);
 
 	bool changestatus(vector<protocolname> &pnames, imstatus &st);
 	int contextmenu(icqcontact *c);
